/**************************************************************************
* iptrkg- log grapher (iptrk - ip tracker)
* Copyright (C) 2003 Jason Nunn <jsno@leal.com.au>
* Darwin, NT, Australia.
* Originally developed for Arafura Connect (Dec 2000 - Mar 2003)
* Donated to GPL before liquidation.
*
* =======================================================================
* produces time domain stats from log_ip logs
* first produced to product daily volume stats for morgan buckley
*
**************************************************************************/
#include "defs.h"
#include "lib.p"

typedef struct
{
  U_32    addr;
  voldr_t voldr;
} bill_vol_t;

PRIVATE FILE    *file_fp;
PRIVATE treap_t *bill_vol_bt;
PRIVATE voldr_t total;
PRIVATE time_t  start_time,
                end_time;

/**************************************************************************
*
**************************************************************************/
PRIVATE void calc_vol(char *s, voldr_t *v)
{
  U_64 x;

  if(!get_next_vol_u64(&s, &x)) return;
  v->I += x;
  total.I += x;

  if(!get_next_vol_u64(&s, &x)) return;
  v->O += x;
  total.O += x;

  if(!get_next_vol_u64(&s, &x)) return;
  v->u += x;
  total.u += x;

  if(!get_next_vol_u64(&s, &x)) return;
  v->i += x;
  total.i += x;

  if(!get_next_vol_u64(&s, &x)) return;
  v->o += x;
  total.o += x;
}

PRIVATE void print_entry(voldr_t *v)
{
  char str1[STR_LEN + 1],
       str2[STR_LEN + 1],
       str3[STR_LEN + 1],
       str4[STR_LEN + 1],
       str5[STR_LEN + 1];

  print_vol_u64(str1, v->I);
  print_vol_u64(str2, v->O);
  print_vol_u64(str3, v->u);
  print_vol_u64(str4, v->i);
  print_vol_u64(str5, v->o);
  printf("%20s %20s %20s %20s %20s\n",
    str1, str2, str3, str4, str5);
}

/**************************************************************************
*
**************************************************************************/
PRIVATE int print_daily_volumes(char *path)
{
  char    str[STR_LEN + 1],
          date[STR_SMALL + 1],
          old_date[STR_SMALL + 1] = "";
  time_t  tm;
  voldr_t v;

  file_fp = fopen(path, "r");
  if(file_fp == NULL)
  {
    log_e(__func__, "::fopen()");
    return 0;
  }

  printf(
    "--\n%-15s %20s %20s %20s %20s %20s\n"
    "--\n",
    "DAY", "I", "O", "u", "i", "o");

  memset(&v, 0, sizeof(voldr_t));
  if(log_ip_fast_find(file_fp, start_time))
    for(;;)
    {
      if(feof(file_fp))
        break;

      if(fgets(str, STR_LEN, file_fp) == NULL)
        break;

      if(str[0] != 'o')
        continue;

      if(sscanf(str, "%*c %lx %s", &tm, date) != 2)
        continue;

      if(tm < start_time || tm > end_time)
        continue;

      if(strcmp(date, old_date))
      {
        if(old_date[0] != 0)
        {
          print_entry(&v);
          memset(&v, 0, sizeof(voldr_t));
        }
        strcpy(old_date, date);
        printf("%-15s ", date);
      }

      if(fgets(str, STR_LEN, file_fp) == NULL)
        break;

      calc_vol(str, &v);
    }

  fclose(file_fp);

  print_entry(&v);
  printf("--\n%-15s ", "TOTAL");
  print_entry(&total);

  return 1;
}

/**************************************************************************
*
**************************************************************************/
PRIVATE void bill_vol_add_entry(U_32 addr, voldr_t *v)
{
  bill_vol_t e;

  int append_cmp_func(void *n1, void *n2)
  {
    return ((bill_vol_t *)n1)->addr - ((bill_vol_t *)n2)->addr;
  }

  e.addr = addr;
  memcpy(&e.voldr, v, sizeof(voldr_t));
  treap_append(&bill_vol_bt, &e, sizeof(bill_vol_t), append_cmp_func);
}

PRIVATE void calc_billing_volumes(char *ip_addr)
{
  char           str[STR_LEN + 1];
  time_t         tm;
  voldr_t        v;
  struct in_addr in;

  memset(&v, 0, sizeof(voldr_t));
  if(log_ip_fast_find(file_fp, start_time))
    for(;;)
    {
      if(feof(file_fp))
        break;

      if(fgets(str, STR_LEN, file_fp) == NULL)
        break;

      if(str[0] != 'o')
      continue;

      if(sscanf(str, "%*c %lx", &tm) != 1)
        continue;

      if(tm < start_time || tm > end_time)
        continue;

      if(fgets(str, STR_LEN, file_fp) == NULL)
        break;

      calc_vol(str, &v);
    }

  inet_aton(ip_addr, &in);
  bill_vol_add_entry(ntohl(in.s_addr), &v);
}

PRIVATE void print_billing_volumes(void)
{
  struct in_addr in;

  void p_entry(treap_t *t)
  {
    bill_vol_t *n;

    if(t == NULL)
      return;

    p_entry(t->l);
    n = TREAP_ENTRY(t);
    in.s_addr = htonl(n->addr);
    printf("%-15s ", inet_ntoa(in));
    print_entry(&(n->voldr));
    p_entry(t->r);
  }

  p_entry(bill_vol_bt);
  printf("--\n%-15s ", "TOTAL");
  print_entry(&total);
}

/**************************************************************************
*
**************************************************************************/
PRIVATE void print_all_billing_volumes(char *path)
{
  DIR           *dp;
  struct dirent *dir_p;
  struct stat   stat_buf;
  char          str[STR_LEN + 1], *s;

  dp = opendir(path);
  if(dp == NULL)
  {
    log_e(__func__, "::opendir()");
    exit(-1);
  }

  strcpy(str, path);
  strcat(str, "/");
  s = str + strlen(str);

  printf(
    "--\n%-15s %20s %20s %20s %20s %20s\n"
    "--\n",
    "ADDRESS", "I", "O", "u", "i", "o");

  for(;;)
  {
    dir_p = readdir(dp);
    if(dir_p == NULL)
      break;

    strcpy(s, dir_p->d_name);
    if(stat(str, &stat_buf) == -1)
    {
      log_e(__func__, "::stat()");
      exit(-1);
    }

    if(S_ISDIR(stat_buf.st_mode))
      continue;

    file_fp = fopen(str, "r");
    if(file_fp != NULL)
    {
      calc_billing_volumes(dir_p->d_name + 4);
      fclose(file_fp);
    }
    else
    {
      log_e(__func__, "::fopen()");
      return;
    }
  }

  closedir(dp);

  print_billing_volumes();
}

/**************************************************************************
*
**************************************************************************/
int main(int argc, char *argv[])
{
  int         x;
  struct stat stat_buf;
  char        ip_log_path[STR_LEN + 1] = IPTRK_LOG_IP_PATHC;
  char        str[STR_LEN + 1];

  welcome();
  timer = end_time = time(NULL);
  for(x = 0; x < argc; x++)
  {
    if(argv[x][0] == '-')
    {
      switch(argv[x][1])
      {
        case 'h':
          printf(
            "Usage: iptrkq_log_ip <options>\n"
            " -h           Display this help info\n"
            " -f <log_ip>  log_ip file/directory to interrogate\n"
            " -t <time>    Start time (Default = Epoch)\n"
            " -n <time>    End time   (Default = Current time)\n"
            "    (Nb/ Time format can be either YYYY:MM:DD:HH:MM or HH:MM)\n"
            "\n"
          );
          return 0;

        case 'f':
          strncpy2(ip_log_path, parse_cmdline(argc, argv, x), STR_LEN);
          break;

        case 't':
          start_time = datestr2time(parse_cmdline(argc, argv, x));
          break;

        case 'n':
          end_time = datestr2time(parse_cmdline(argc, argv, x));
          break;
      }
    }
  }

  if(stat(ip_log_path, &stat_buf) == -1)
  {
    log_e(__func__, "::stat()");
    exit(-1);
  }

  printf(
    "ip_log path: %s\n", ip_log_path);
  format_date_entry(start_time, str);
  printf(
    " start time: %s\n", str);
  format_date_entry(end_time, str);
  printf(
    "   end time: %s\n\n", str);

  if(S_ISDIR(stat_buf.st_mode))
    print_all_billing_volumes(ip_log_path);
  else
    if(!print_daily_volumes(ip_log_path))
      exit(-1);

  printf("--\nEND\n");

  return 0;
}
